@@ -2,7 +2,7 @@ package App::perlbrew;
 use strict;
 use warnings;
 use 5.008;
-our $VERSION = "0.61";
+our $VERSION = "0.62";
 
 use Config;
 use Getopt::Long ();
@@ -29,11 +29,6 @@ sub rmpath {
     File::Path::rmtree([@_], 0, 1);
 }
 
-sub uniq(@) {
-    my %a;
-    grep { ++$a{$_} == 1 } @_;
-}
-
 sub min(@) {
     my @a = @_;
     my $m = $a[0];
@@ -203,6 +198,13 @@ sub current_perl_executable {
     return $self->installed_perl_executable($self->current_perl);
 }
 
+sub current_env {
+    my ($self) = @_;
+    my $l = $self->current_lib;
+    $l = "@" . $l if $l;
+    return $self->current_perl . $l;
+}
+
 sub installed_perl_executable {
     my ($self, $name) = @_;
     my $executable = catfile($self->root, "perls", $name, "bin", "perl");
@@ -1203,6 +1205,12 @@ sub is_installed {
     return grep { $name eq $_->{name} } $self->installed_perls;
 }
 
+sub assert_installation {
+    my ($self, $name) = @_;
+    return 1 if $self->is_installed($name);
+    die "ERROR: The installation \"$name\" is unknown\n\n";
+}
+
 # Return a hash of PERLBREW_* variables
 sub perlbrew_env {
     my ($self, $name) = @_;
@@ -1637,16 +1645,21 @@ sub run_command_exec {
     shift @ARGV; # "exec"
     $self->parse_cmdline (\%opts, @command_options);
 
-    my @exec_with = map { ($_, @{$_->{libs}}) } $self->installed_perls;
-
+    my @exec_with;
     if ($opts{with}) {
+        my %installed = map { $_->{name} => $_ } map { ($_, @{$_->{libs}}) } $self->installed_perls;
+
         my $d = ($opts{with} =~ / /) ? qr( +) : qr(,+);
-        my %x = map { $_ => 1 } grep { $_ } map {
+        my @with = grep { $_ } map {
             my ($p,$l) = $self->resolve_installation_name($_);
             $p .= "\@$l" if $l;
             $p;
         } split $d, $opts{with};
-        @exec_with = grep { $x{ $_->{name} } } @exec_with;
+
+        @exec_with = map { $installed{$_} } @with;
+    }
+    else {
+        @exec_with = map { ($_, @{$_->{libs}}) } $self->installed_perls;
     }
 
     if (0 == @exec_with) {
@@ -1661,6 +1674,7 @@ sub run_command_exec {
         local @ENV{ keys %env } = values %env;
         local $ENV{PATH}    = join(':', $env{PERLBREW_PATH}, $ENV{PATH});
         local $ENV{MANPATH} = join(':', $env{PERLBREW_MANPATH}, $ENV{MANPATH}||"");
+        local $ENV{PERL5LIB} = $env{PERL5LIB} || "";
 
         print "$i->{name}\n==========\n" unless $self->{quiet};
         $self->do_system(@ARGV);
@@ -1698,10 +1712,6 @@ USAGE
         return;
     }
 
-    unless ( $self->is_installed($name) ) {
-        die "\nABORT: The installation `${name}` does not exist.\n\n";
-    }
-
     my $path_name  = catfile($self->root, "perls", $name);
     my $path_alias = catfile($self->root, "perls", $alias) if $alias;
 
@@ -1710,6 +1720,8 @@ USAGE
     }
 
     if ($cmd eq 'create') {
+        $self->assert_known_installation($name);
+
         if ( $self->is_installed($alias) && !$self->{force} ) {
             die "\nABORT: The installation `${alias}` already exists. Cannot override.\n\n";
         }
@@ -1719,6 +1731,8 @@ USAGE
         symlink($path_name, $path_alias);
     }
     elsif($cmd eq 'delete') {
+        $self->assert_known_installation($name);
+
         unless (-l $path_name) {
             die "\nABORT: The installation name `$name` is not an alias, cannot remove.\n\n";
         }
@@ -1726,6 +1740,8 @@ USAGE
         unlink($path_name);
     }
     elsif($cmd eq 'rename') {
+        $self->assert_known_installation($name);
+
         unless (-l $path_name) {
             die "\nABORT: The installation name `$name` is not an alias, cannot rename.\n\n";
         }
@@ -1736,6 +1752,9 @@ USAGE
 
         rename($path_name, $path_alias);
     }
+    elsif($cmd eq 'help') {
+        $self->run_command_help("alias");
+    }
     else {
         die "\nERROR: Unrecognized action: `${cmd}`.\n\n";
     }
@@ -1924,14 +1943,14 @@ sub run_command_upgrade_perl {
 
 sub run_command_list_modules {
     my ($self) = @_;
-
-    $self->{quiet} = 1;
-    $self->{original_argv} = [
-        "exec", "--with", $self->current_perl,
-        'perl', '-MExtUtils::Installed', '-le', 'BEGIN{@INC=grep(!/^\.$/,@INC)}; print for ExtUtils::Installed->new->modules'
-    ];
-
-    $self->run_command_exec();
+    my $class = ref($self) || __PACKAGE__;
+    my $app = $class->new(
+        qw(--quiet exec --with),
+        $self->current_env,
+        'perl', '-MExtUtils::Installed', '-le',
+        'BEGIN{@INC=grep {$_ ne q!.!} @INC}; print for ExtUtils::Installed->new->modules;'
+    );
+    $app->run;
 }
 
 sub resolve_installation_name {
@@ -1961,7 +1980,7 @@ sub run_command_info {
 
     print "Current perl:";
     if ($self->current_perl) {
-        print "  Name: " . $self->current_perl . ($self->current_lib && "@".$self->current_lib);
+        print "  Name: " . $self->current_env;
         print "  Path: " . $self->current_perl_executable;
     }
     else {
@@ -2029,7 +2048,7 @@ __perlbrew_reinit() {
     fi
 
     echo '# DO NOT EDIT THIS FILE' > "$PERLBREW_HOME/init"
-    command perlbrew env $1 | grep PERLBREW_ >> "$PERLBREW_HOME/init"
+    command perlbrew env $1 | \grep PERLBREW_ >> "$PERLBREW_HOME/init"
     . "$PERLBREW_HOME/init"
     __perlbrew_set_path
 }
